@@ -95,13 +95,17 @@ class fm_tx_graph (stdgui.gui_flow_graph):
 
         # determine the daughterboard subdevice we're using
         if options.tx_subdev_spec is None:
-            options.tx_subdev_spec = pick_tx_subdevice(self.u)
+            options.tx_subdev_spec = usrp.pick_tx_subdevice(self.u)
+
+        m = usrp.determine_tx_mux_value(self.u, options.tx_subdev_spec)
+        #print "mux = %#04x" % (m,)
+        self.u.set_mux(m)
         self.subdev = usrp.selected_subdev(self.u, options.tx_subdev_spec)
         print "Using TX d'board %s" % (self.subdev.side_and_name(),)
 
         self.subdev.set_gain(self.subdev.gain_range()[1])    # set max Tx gain
-
         self.set_freq(options.freq)
+        self.subdev.set_enable(True)                         # enable transmitter
 
         sum = gr.add_cc ()
 
@@ -147,52 +151,18 @@ class fm_tx_graph (stdgui.gui_flow_graph):
         any residual_freq to the s/w freq translater.
         """
 
-        r = usrp.tune(self.u, 0, self.subdev, target_freq)
+        r = self.u.tune(self.subdev._which, self.subdev, target_freq)
         if r:
             print "r.baseband_freq =", eng_notation.num_to_str(r.baseband_freq)
             print "r.dxc_freq      =", eng_notation.num_to_str(r.dxc_freq)
             print "r.residual_freq =", eng_notation.num_to_str(r.residual_freq)
             print "r.inverted      =", r.inverted
             
-            # Use residual_freq in s/w freq translator
+            # Could use residual_freq in s/w freq translator
             return True
 
         return False
 
-
-def pick_tx_subdevice(u):
-    """
-    The user didn't specify a tx subdevice on the command line.
-    Try for one of these, in order: FLEX_400, FLEX_900, FLEX_1200, FLEX_2400,
-    BASIC_TX, whatever's on side A.
-
-    @return a subdev_spec
-    """
-    return pick_subdev(u, (usrp_dbid.FLEX_400_TX,
-                           usrp_dbid.FLEX_900_TX,
-                           usrp_dbid.FLEX_1200_TX,
-                           usrp_dbid.FLEX_2400_TX,
-                           usrp_dbid.BASIC_TX))
-
-
-def pick_subdev(u, candidates):
-    """
-    @param u:          usrp instance
-    @param candidates: list of dbids
-    @returns: subdev specification
-    """
-    db0 = u.db[0][0].dbid()
-    db1 = u.db[1][0].dbid()
-    for c in candidates:
-        if c == db0: return (0, 0)
-        if c == db1: return (1, 0)
-    if db0 >= 0:
-        return (0, 0)
-    if db1 >= 0:
-        return (1, 0)
-    raise RuntimeError, "No suitable daughterboard found!"
-
-
 def main ():
     app = stdgui.stdapp (fm_tx_graph, "Multichannel FM Tx")
     app.MainLoop ()
