@@ -30,6 +30,20 @@ import wx
 import sys
 
 
+def pick_subdevice(u):
+    """
+    The user didn't specify a subdevice on the command line.
+    If there's a daughterboard on A, select A.
+    If there's a daughterboard on B, select B.
+    Otherwise, select A.
+    """
+    if u.db[0][0].dbid() >= 0:       # dbid is < 0 if there's no d'board or a problem
+        return (0, 0)
+    if u.db[1][0].dbid() >= 0:
+        return (1, 0)
+    return (0, 0)
+
+
 class app_flow_graph(stdgui.gui_flow_graph):
     def __init__(self, frame, panel, vbox, argv):
         stdgui.gui_flow_graph.__init__(self)
@@ -38,8 +52,8 @@ class app_flow_graph(stdgui.gui_flow_graph):
         self.panel = panel
         
         parser = OptionParser(option_class=eng_option)
-        parser.add_option("-R", "--rx-subdev-spec", type="subdev", default=(0, 0),
-                          help="select USRP Rx side A or B (default=A)")
+        parser.add_option("-R", "--rx-subdev-spec", type="subdev", default=None,
+                          help="select USRP Rx side A or B (default=first one with a daughterboard)")
         parser.add_option("-d", "--decim", type="int", default=16,
                           help="set fgpa decimation rate to DECIM [default=%default]")
         parser.add_option("-f", "--freq", type="eng_float", default=None,
@@ -62,6 +76,8 @@ class app_flow_graph(stdgui.gui_flow_graph):
         # build the graph
 
         self.u = usrp.source_c(decim_rate=options.decim)
+        if options.rx_subdev_spec is None:
+            options.rx_subdev_spec = pick_subdevice(self.u)
         self.u.set_mux(usrp.determine_rx_mux_value(self.u, options.rx_subdev_spec))
 
         if options.width_8:
@@ -77,13 +93,17 @@ class app_flow_graph(stdgui.gui_flow_graph):
 
         input_rate = self.u.adc_freq() / self.u.decim_rate()
 
+        fft_rate = 15
+
         if options.waterfall:
             self.scope = \
-              waterfallsink.waterfall_sink_c (self, panel, fft_size=1024, sample_rate=input_rate)
+              waterfallsink.waterfall_sink_c (self, panel, fft_size=1024, sample_rate=input_rate,
+                                              fft_rate=fft_rate)
         elif options.oscilloscope:
             self.scope = scopesink.scope_sink_c(self, panel, sample_rate=input_rate)
         else:
-            self.scope = fftsink.fft_sink_c (self, panel, fft_size=1024, sample_rate=input_rate)
+            self.scope = fftsink.fft_sink_c (self, panel, fft_size=1024, sample_rate=input_rate,
+                                             fft_rate=fft_rate)
 
         self.connect(self.u, self.scope)
 
@@ -103,14 +123,15 @@ class app_flow_graph(stdgui.gui_flow_graph):
 
         self.set_gain(options.gain)
 
-        if not(self.set_freq(options.freq)):
-            self._set_status_msg("Failed to set initial frequency")
-
         if self.show_debug_info:
             self.myform['decim'].set_value(self.u.decim_rate())
             self.myform['fs@usb'].set_value(self.u.adc_freq() / self.u.decim_rate())
             self.myform['dbname'].set_value(self.subdev.name())
-                        
+            self.myform['baseband'].set_value(0)
+            self.myform['ddc'].set_value(0)
+
+        if not(self.set_freq(options.freq)):
+            self._set_status_msg("Failed to set initial frequency")
 
     def _set_status_msg(self, msg):
         self.frame.GetStatusBar().SetStatusText(msg, 0)
@@ -196,7 +217,7 @@ class app_flow_graph(stdgui.gui_flow_graph):
         the result of that operation and our target_frequency to
         determine the value for the digital down converter.
         """
-        r = usrp.tune(self.u, 0, self.subdev, target_freq)
+        r = self.u.tune(0, self.subdev, target_freq)
         
         if r:
             self.myform['freq'].set_value(target_freq)     # update displayed value
