@@ -35,6 +35,10 @@ import usrp_dbid
 
 from Numeric import convolve, array
 
+#import os
+#print "pid =", os.getpid()
+#raw_input('Press Enter to continue: ')
+
 # ////////////////////////////////////////////////////////////////////////
 #                           Control Stuff
 # ////////////////////////////////////////////////////////////////////////
@@ -304,6 +308,7 @@ class transmit_path(gr.hier_block):
         # determine the daughterboard subdevice we're using
         if subdev_spec is None:
             subdev_spec = usrp.pick_tx_subdevice(self.u)
+        self.u.set_mux(usrp.determine_tx_mux_value(self.u, subdev_spec))
         self.subdev = usrp.selected_subdev(self.u, subdev_spec)
         print "TX using", self.subdev.name()
 
@@ -328,8 +333,7 @@ class transmit_path(gr.hier_block):
         determine the value for the digital up converter.  Finally, we feed
         any residual_freq to the s/w freq translater.
         """
-
-        r = usrp.tune(self.u, 0, self.subdev, target_freq)
+        r = self.u.tune(self.subdev._which, self.subdev, target_freq)
         if r:
             # Use residual_freq in s/w freq translator
             return True
@@ -461,8 +465,7 @@ class receive_path(gr.hier_block):
         FPGA.  Finally, we feed any residual_freq to the s/w freq
         translator.
         """
-
-        r = usrp.tune(self.u, 0, self.subdev, target_freq)
+        r = self.u.tune(0, self.subdev, target_freq)
         if r:
             # Use residual_freq in s/w freq translater
             # print "residual_freq =", r.residual_freq
