@@ -3,15 +3,15 @@
 from gnuradio import gr, gru
 from gnuradio import usrp
 from gnuradio.eng_option import eng_option
+from gnuradio import eng_notation
 from optparse import OptionParser
+import sys
 
 
-class siggen (object):
-    __slots__ = ['interp', 'waveform_type', 'waveform_ampl',
-                 'waveform_freq', 'waveform_offset', 'fg', 'usrp',
-                 'siggen', 'noisegen', 'src', 'file_sink' ]
-
+class my_graph(gr.flow_graph):
     def __init__ (self):
+        gr.flow_graph.__init__(self)
+        
         # controllable values
         self.interp = 64
         self.waveform_type = gr.GR_SIN_WAVE
@@ -21,14 +21,8 @@ class siggen (object):
         self._instantiate_blocks ()
         self.set_waveform_type (self.waveform_type)
 
-    def start (self):
-        self.fg.start ()
-
-    def stop (self):
-        self.fg.stop ()
-        
     def usb_freq (self):
-        return self.usrp.dac_freq() / self.interp
+        return self.u.dac_freq() / self.interp
 
     def usb_throughput (self):
         return self.usb_freq () * 4
@@ -57,15 +51,11 @@ class siggen (object):
     def set_interpolator (self, interp):
         self.interp = interp
         self.siggen.set_sampling_freq (self.usb_freq ())
-        self.usrp.set_interp_rate (interp)
+        self.u.set_interp_rate (interp)
 
-    def set_duc_freq (self, freq):
-        self.usrp.set_tx_freq (0, freq)
-        
     def _instantiate_blocks (self):
-        self.fg = gr.flow_graph ()
         self.src = None
-        self.usrp = usrp.sink_c (0, self.interp)
+        self.u = usrp.sink_c (0, self.interp)
         
         self.siggen = gr.sig_source_c (self.usb_freq (),
                                        gr.GR_SIN_WAVE,
@@ -79,27 +69,57 @@ class siggen (object):
         # self.file_sink = gr.file_sink (gr.sizeof_gr_complex, "siggen.dat")
 
     def _configure_graph (self, type):
-        was_running = self.fg.is_running ()
+        was_running = self.is_running ()
         if was_running:
-            self.fg.stop ()
-        self.fg.disconnect_all ()
+            self.stop ()
+        self.disconnect_all ()
         if type == gr.GR_SIN_WAVE or type == gr.GR_CONST_WAVE:
-            self.fg.connect (self.siggen, self.usrp)
-            # self.fg.connect (self.siggen, self.file_sink)
+            self.connect (self.siggen, self.u)
+            # self.connect (self.siggen, self.file_sink)
             self.siggen.set_waveform (type)
             self.src = self.siggen
         elif type == gr.GR_UNIFORM or type == gr.GR_GAUSSIAN:
-            self.fg.connect (self.noisegen, self.usrp)
+            self.connect (self.noisegen, self.u)
             self.noisegen.set_type (type)
             self.src = self.noisegen
         else:
             raise ValueError, type
         if was_running:
-            self.fg.start ()
+            self.start ()
+
+    def set_freq(self, target_freq):
+        """
+        Set the center frequency we're interested in.
+
+        @param target_freq: frequency in Hz
+        @rypte: bool
+
+        Tuning is a two step process.  First we ask the front-end to
+        tune as close to the desired frequency as it can.  Then we use
+        the result of that operation and our target_frequency to
+        determine the value for the digital up converter.
+        """
+        r = self.u.tune(self.subdev._which, self.subdev, target_freq)
+        if r:
+            print "r.baseband_freq =", eng_notation.num_to_str(r.baseband_freq)
+            print "r.dxc_freq      =", eng_notation.num_to_str(r.dxc_freq)
+            print "r.residual_freq =", eng_notation.num_to_str(r.residual_freq)
+            print "r.inverted      =", r.inverted
+            return True
+
+        return False
+
 
 
 def main ():
     parser = OptionParser (option_class=eng_option)
+    parser.add_option ("-T", "--tx-subdev-spec", type="subdev", default=(0, 0),
+                       help="select USRP Tx side A or B")
+    parser.add_option ("-f", "--rf-freq", type="eng_float", default=None,
+                       help="set RF center frequency to FREQ")
+    parser.add_option ("-i", "--interp", type="int", default=64,
+                       help="set fgpa interpolation rate to INTERP")
+
     parser.add_option ("--sine", dest="type", action="store_const", const=gr.GR_SIN_WAVE,
                        help="generate a complex sinusoid [default]", default=gr.GR_SIN_WAVE)
     parser.add_option ("--const", dest="type", action="store_const", const=gr.GR_CONST_WAVE, 
@@ -109,36 +129,52 @@ def main ():
     parser.add_option ("--uniform", dest="type", action="store_const", const=gr.GR_UNIFORM,
                        help="generate Uniform random output")
 
-    parser.add_option ("-i", "--interp", type="int", default=64,
-                       help="set fgpa interpolation rate to INTERP")
-    parser.add_option ("-f", "--freq", type="eng_float", default=100e3,
+    parser.add_option ("-w", "--waveform-freq", type="eng_float", default=100e3,
                        help="set waveform frequency to FREQ")
     parser.add_option ("-a", "--amplitude", type="eng_float", default=16e3,
                        help="set waveform amplitude to AMPLITUDE", metavar="AMPL")
     parser.add_option ("-o", "--offset", type="eng_float", default=0,
                        help="set waveform offset to OFFSET")
-    parser.add_option ("-c", "--duc-freq", type="eng_float", default=0,
-                       help="set Tx DUC frequency to FREQ", metavar="FREQ")
-    parser.add_option ("-n", "--nchannels", type="int", default=1,
-                       help="set number of output channels to NCHANNELS")
-    parser.add_option ("-m", "--mux", type="intx", default=0x98,
-                       help="set output mux register")
     (options, args) = parser.parse_args ()
 
-    sg = siggen ()
-    sg.set_interpolator (options.interp)
-    sg.set_waveform_type (options.type)
-    sg.set_waveform_freq (options.freq)
-    sg.set_waveform_ampl (options.amplitude)
-    sg.set_waveform_offset (options.offset)
-    sg.set_duc_freq (options.duc_freq)
+    if len(args) != 0:
+        parser.print_help()
+        raise SystemExit
 
-    sg.usrp.set_nchannels (options.nchannels)
-    sg.usrp.set_mux (gru.hexint(options.mux))
+    if options.rf_freq is None:
+        sys.stderr.write("usrp_siggen: must specify RF center frequency with -f RF_FREQ\n")
+        parser.print_help()
+        raise SystemExit
 
-    sg.start ()
-    raw_input ('Press Enter to quit: ')
-    sg.stop ()
+    fg = my_graph()
+    fg.set_interpolator (options.interp)
+    fg.set_waveform_type (options.type)
+    fg.set_waveform_freq (options.waveform_freq)
+    fg.set_waveform_ampl (options.amplitude)
+    fg.set_waveform_offset (options.offset)
+
+    # determine the daughterboard subdevice we're using
+    if options.tx_subdev_spec is None:
+        options.tx_subdev_spec = usrp.pick_tx_subdevice(fg.u)
+
+    m = usrp.determine_tx_mux_value(fg.u, options.tx_subdev_spec)
+    #print "mux = %#04x" % (m,)
+    fg.u.set_mux(m)
+    fg.subdev = usrp.selected_subdev(fg.u, options.tx_subdev_spec)
+    print "Using TX d'board %s" % (fg.subdev.side_and_name(),)
+    
+    fg.subdev.set_gain(fg.subdev.gain_range()[1])    # set max Tx gain
+
+    if not fg.set_freq(options.rf_freq):
+        sys.stderr.write('Failed to set RF frequency\n')
+        raise SystemExit
+    
+    fg.subdev.set_enable(True)                       # enable transmitter
+
+    try:
+        fg.run()
+    except KeyboardInterrupt:
+        pass
 
 if __name__ == '__main__':
     main ()
